Package: spicyR
Type: Package
Title: Spatial analysis of in situ cytometry data
Version: 1.22.0
Authors@R: c(
    person("Nicolas", "Canete", , "nicolas.canete@sydney.edu.au", role = "aut"),
    person("Ellis", "Patrick", , "ellis.patrick@sydney.edu.au", role = c("aut", "cre")),
    person("Nicholas", "Robertson", , "nicholas.robertson@sydney.edu.au", role = "ctb"),
    person("Alex", "Qin", , "alex.qin@sydney.edu.au", role = "ctb"),
    person("Farhan", "Ameen", ,"farhan.ameen@sydney.edu.au", role = "ctb"),
    person("Shreya", "Rao", ,"shreya.rajeshrao@sydney.edu.au", role = "ctb")
  )
Description: The spicyR package provides a framework for performing inference on changes in spatial relationships between pairs of cell types for cell-resolution spatial omics technologies. spicyR consists of three primary steps: (i) summarizing the degree of spatial localization between pairs of cell types for each image; (ii) modelling the variability in localization summary statistics as a function of cell counts and (iii) testing for changes in spatial localizations associated with a response variable.  
License: GPL (>=2)
LazyData: true
biocViews: SingleCell, CellBasedAssays, Spatial
Encoding: UTF-8
Depends: R (>= 4.1)
VignetteBuilder: knitr
BugReports: https://github.com/SydneyBioX/spicyR/issues
URL: https://ellispatrick.github.io/spicyR/
        https://github.com/SydneyBioX/spicyR,
        https://sydneybiox.github.io/spicyR/
Imports: BiocParallel, ClassifyR, S4Vectors, SingleCellExperiment,
        SpatialExperiment, SummarizedExperiment, cli, concaveman,
        coxme, data.table, dplyr, ggforce, ggh4x, ggnewscale, ggplot2,
        ggthemes, grDevices, lifecycle, lmerTest, magrittr, methods,
        pheatmap, rlang, scales, scam, simpleSeg, spatstat.explore,
        spatstat.geom, stats, survival, tibble, tidyr
Suggests: SpatialDatasets, BiocStyle, knitr, rmarkdown, pkgdown,
        imcRtools, testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/spicyR
git_branch: RELEASE_3_22
git_last_commit: a906b18
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:46:40 UTC; biocbuild
Author: Nicolas Canete [aut],
  Ellis Patrick [aut, cre],
  Nicholas Robertson [ctb],
  Alex Qin [ctb],
  Farhan Ameen [ctb],
  Shreya Rao [ctb]
Maintainer: Ellis Patrick <ellis.patrick@sydney.edu.au>
