# Generated by roxygen2: do not edit by hand

export(bind)
export(colTest)
export(convPairs)
export(getPairwise)
export(getProp)
export(imageCrossPlot)
export(plotImage)
export(signifPlot)
export(spicy)
export(spicyBoxPlot)
export(topPairs)
exportClasses(SpicyResults)
import(SpatialExperiment)
import(SummarizedExperiment)
import(dplyr)
import(ggh4x)
import(ggplot2)
import(ggthemes)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(ClassifyR,colCoxTests)
importFrom(S4Vectors,as.data.frame)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,colPair)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,colData)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(concaveman,concaveman)
importFrom(coxme,coxme)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,CJ)
importFrom(data.table,as.data.table)
importFrom(data.table,setkey)
importFrom(dplyr,across)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(ggforce,geom_arc_bar)
importFrom(ggforce,geom_circle)
importFrom(ggnewscale,new_scale)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_axis)
importFrom(ggplot2,guide_colourbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_gradient2)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grid,gpar)
importFrom(grid,grobTree)
importFrom(grid,polygonGrob)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_warn)
importFrom(lmerTest,lmer)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(rlang,.data)
importFrom(scales,squish)
importFrom(scam,scam)
importFrom(spatstat.explore,density.ppp)
importFrom(spatstat.geom,area)
importFrom(spatstat.geom,border)
importFrom(spatstat.geom,closepairs)
importFrom(spatstat.geom,convexhull)
importFrom(spatstat.geom,discs)
importFrom(spatstat.geom,inside.owin)
importFrom(spatstat.geom,intersect.owin)
importFrom(spatstat.geom,nearest.valid.pixel)
importFrom(spatstat.geom,owin)
importFrom(spatstat.geom,ppp)
importFrom(spatstat.geom,solapply)
importFrom(spatstat.geom,union.owin)
importFrom(stats,lm)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,t.test)
importFrom(stats,weights)
importFrom(stats,wilcox.test)
importFrom(survival,coxph)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,separate)
