% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy.R
\name{bind}
\alias{bind}
\title{Produces a dataframe showing L-function metric for each imageID entry.}
\usage{
bind(results, pairName = NULL)
}
\arguments{
\item{results}{Spicy test result obtained from spicy.}

\item{pairName}{A string specifying the pairwise interaction of interest. If NULL, all
pairwise interactions are shown.}
}
\value{
A data.frame containing the colData related to the results.
}
\description{
Produces a dataframe showing L-function metric for each imageID entry.
}
\examples{

data(spicyTest)
df <- bind(spicyTest)

}
