% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicyBoxPlot.R
\name{spicyBoxPlot}
\alias{spicyBoxPlot}
\title{Plots boxplot for a specified cell-cell relationship}
\usage{
spicyBoxPlot(results, from = NULL, to = NULL, rank = NULL)
}
\arguments{
\item{results}{Data frame obtained from spicy.}

\item{from}{Cell type which you would like to compare to the to cell type.}

\item{to}{Cell type which you would like to compare to the from cell type.}

\item{rank}{Ranking of cell type in terms of p-value, the smaller the p-value
the higher the rank.}
}
\value{
a ggplot2 boxplot
}
\description{
Plots boxplot for a specified cell-cell relationship
}
\examples{
data(spicyTest)

spicyBoxPlot(spicyTest,
             rank = 1)

}
