% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy.R
\name{getProp}
\alias{getProp}
\title{Get proportions from a SummarizedExperiment.}
\usage{
getProp(cells, feature = "cellType", imageID = "imageID")
}
\arguments{
\item{cells}{A SingleCellExperiment, SpatialExperiment or data.frame.}

\item{feature}{The feature of interest}

\item{imageID}{The imageID's}
}
\value{
Proportions
}
\description{
Get proportions from a SummarizedExperiment.
}
\examples{
data("diabetesData")
prop <- getProp(diabetesData)
}
