% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{diabetesData}
\alias{diabetesData}
\title{Diabetes IMC data in SCE format.}
\format{
diabetesData_SCE a SingleCellExperiment object
}
\usage{
data("diabetesData")
}
\description{
This is a subset of the Damond et al 2019 imaging mass cytometry dataset. The data 
contains cells in the pancreatic islets of individuals with early onset 
diabetes and healthy controls. 
The object contains single-cell data of 160 images from 8 subjects, 
with 20 images per subject.
}
\details{
Converted into a SingleCellExperiment format.
}
\keyword{datasets}
