% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{spicyTest}
\alias{spicyTest}
\title{Results from spicy for diabetesData}
\format{
spicyTest a spicy object
}
\usage{
data("spicyTest")
}
\description{
Results from the call:
spicyTest <- spicy(diabetesData, 
                   condition = "condition", 
                   subject = "subject")
}
\keyword{datasets}
