% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{topPairs}
\alias{topPairs}
\alias{topPairs,SpicyResults-method}
\title{A table of the significant results from spicy tests}
\usage{
topPairs(x, coef = NULL, n = 10, adj = "fdr", cutoff = NULL, figures = NULL)
}
\arguments{
\item{x}{The output from spicy.}

\item{coef}{Which coefficient to list.}

\item{n}{Extract the top n most significant pairs.}

\item{adj}{Which p-value adjustment method to use, argument for p.adjust().}

\item{cutoff}{A p-value threshold to extract significant pairs.}

\item{figures}{Round to `figures` significant figures.}
}
\value{
A data.frame
}
\description{
A table of the significant results from spicy tests
}
\examples{

data(spicyTest)
topPairs(spicyTest)

}
