% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signifPlot.R
\name{survBubble}
\alias{survBubble}
\title{Plots survival results from spicy.}
\usage{
survBubble(
  result,
  fdr = FALSE,
  cutoff = 0.05,
  colourGradient = c("#4575B4", "white", "#D73027"),
  marksToPlot = NULL,
  contextColours = NULL,
  contextLabels = waiver()
)
}
\arguments{
\item{result}{A spicyResults object that contains survival results.}

\item{fdr}{TRUE if FDR correction is used.}

\item{cutoff}{Significance threshold for circles in bubble plot.}

\item{colourGradient}{A vector of colours, used to define the low, medium, and high values for the colour scale.}

\item{marksToPlot}{Vector of marks to include in bubble plot.}

\item{contextColours}{Used for \code{\link[Statial]{Kontextual}} results. A named list specifying the colours for each context.
By default the Tableau colour palette is used.}

\item{contextLabels}{Used for \code{\link[Statial]{Kontextual}} results. A named list to change the default labels for each context.}
}
\value{
A ggplot object.
}
\description{
Plots survival results from spicy.
}
